package fr.ippon.kafkaLatency.producer;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import fr.ippon.kafkaLatency.utils.Config;
import fr.ippon.kafkaLatency.utils.MetricsProducerReporter;
import org.apache.kafka.clients.admin.AdminClient;
import org.apache.kafka.clients.admin.CreateTopicsResult;
import org.apache.kafka.clients.admin.NewTopic;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.errors.TopicExistsException;
import org.apache.kafka.common.serialization.StringSerializer;
import org.apache.kafka.connect.json.JsonSerializer;
import org.apache.log4j.Logger;

import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Properties;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;

/**
 * IPPON 2018.
 */
public class AlimKafkaAsync {

    private static final ClassLoader classLoader = AlimKafkaAsync.class.getClassLoader();

    private final static Logger logger = Logger.getLogger(AlimKafkaAsync.class);

    public static void main(String[] args) throws InterruptedException {

        StringBuilder stats = new StringBuilder();

        Properties props = new Properties();

        Producer<String, JsonNode> producer = null;

        String brokers = Config.getProperty("bootstrap.servers");
        String topic = Config.getProperty("topic");
        String numPartitions = Config.getProperty("numPartitions");
        String replication = Config.getProperty("replication");
        int total = Integer.valueOf(Config.getProperty("stats.total"));

        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, brokers);

        //Create topic if not exist
        AdminClient adminClient = AdminClient.create(props);

        NewTopic newTopic = new NewTopic(topic, Integer.valueOf(numPartitions), Integer.valueOf(replication).shortValue());
        CreateTopicsResult createTopicsResult = adminClient.createTopics(Collections.singleton(newTopic));

        try {
            createTopicsResult.all().get();
        } catch (ExecutionException e) {
            if (e.getCause() instanceof TopicExistsException) {
                logger.info("Topic already exists (delete manually to change settings) !!");
            } else if (e.getCause() instanceof TimeoutException) {
                logger.error("Timeout !!");
            } else {
                logger.error("Error !!" + e.getCause());
            }
        } finally {
            adminClient.close();
        }

        try {

            props.put(ProducerConfig.CLIENT_ID_CONFIG, "KafkaProducerAsync");
            props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG,
                    StringSerializer.class.getName());
            props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG,
                    JsonSerializer.class.getName());

            ObjectMapper mapper = new ObjectMapper();

            producer = new KafkaProducer<String, JsonNode>(props);
            File file = new File(Objects.requireNonNull(classLoader.getResource("listPersons.json")).getFile());

            long i = 0;

            List<JsonNode> allNodes = mapper.readValue(file, new TypeReference<List<JsonNode>>() {
            });

            Producer<String, JsonNode> finalProducer = producer;

            while (i < total) {
                LocalDateTime start = LocalDateTime.now();

                long size = allNodes.stream().map(node -> {
                    try {
                        // synchronous API
                        finalProducer.send(new ProducerRecord<String, JsonNode>(topic, node));
                    } catch (Exception e) {
                        logger.error("Error IO" + e.getMessage());
                    }
                    return null;
                }).count();

                i = i + size;

                LocalDateTime end = LocalDateTime.now();

                long millis = ChronoUnit.MILLIS.between(start, end);

                stats.append(allNodes.size()).append(" messages inserted in ").append(millis).append(" ms.").append(System.lineSeparator());
            }
        } catch (IOException e) {
            logger.error("Error IO" + e.getMessage());
        } finally {
            logger.info(stats);
            MetricsProducerReporter metrics = new MetricsProducerReporter(producer);
            metrics.displayMetrics();
            assert producer != null;
            producer.close();
        }
    }

}
