package fr.ippon.kafkaLatency.streaming;

import com.fasterxml.jackson.databind.ObjectMapper;
import fr.ippon.kafkaLatency.model.Person;
import fr.ippon.kafkaLatency.utils.Config;
import fr.ippon.kafkaLatency.utils.UtilStats;
import org.apache.flink.api.common.functions.MapFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.streaming.api.TimeCharacteristic;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.AssignerWithPunctuatedWatermarks;
import org.apache.flink.streaming.api.watermark.Watermark;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaConsumer010;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.log4j.Logger;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Properties;
import java.util.TimeZone;

/**
 * Ippon Tech 2018.
 */
public class KafkaFlink {

    private final static Logger logger = Logger.getLogger(KafkaFlink.class);

    private static final ObjectMapper mapper = new ObjectMapper();

    public static class TimestampsFetcher implements AssignerWithPunctuatedWatermarks<String> {
        final UtilStats stats = new UtilStats("KafkaFlink");

        @Override
        public long extractTimestamp(String element, long previousElementTimestamp) {

            LocalDateTime parsedDate = LocalDateTime.ofInstant(Instant.ofEpochMilli(previousElementTimestamp), TimeZone
                    .getDefault().toZoneId());
            LocalDateTime now = LocalDateTime.now();

            Long millis = ChronoUnit.MILLIS.between(parsedDate, now);

            stats.treatStats(millis);

            return previousElementTimestamp;
        }

        @Override
        public Watermark checkAndGetNextWatermark(String consumerRecord, long l) {
            return new Watermark(l);
        }
    }

    public static void main(String[] args) {
        try {
            String brokers = Config.getProperty("bootstrap.servers");
            String topic = Config.getProperty("topic");
            Integer numPartitions = Config.getIntProperty("numPartitions");

            // set up the streaming execution environment
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
            env.setStreamTimeCharacteristic(TimeCharacteristic.EventTime);
            env.enableCheckpointing(150);
            Properties properties = new Properties();
            properties.setProperty("bootstrap.servers", brokers);
            properties.setProperty("auto.offset.reset", "latest");

            properties.setProperty(ConsumerConfig.GROUP_ID_CONFIG, "Flink");

            FlinkKafkaConsumer010<String> myConsumer = new FlinkKafkaConsumer010<>(topic, new SimpleStringSchema(), properties);

            DataStream<String> input = env.addSource(myConsumer).setParallelism(numPartitions);

            myConsumer.assignTimestampsAndWatermarks(new TimestampsFetcher());

            input.map((MapFunction<String, Person>) (String value) ->
                    mapper.readValue(value, Person.class)
            ).setParallelism(numPartitions).countWindowAll(1000);

            // execute program
            env.execute("Flink Streaming Kafka");
        } catch (Exception e) {
            logger.error("Error " + e.getMessage());
        }
    }

}